/*
-- Company: 		Trenz Electronic
-- Engineer: 		Oleksandr Kiyenko / John Hartfiel
 */
#include "te_si534x.h"

#ifdef CLOCK_SI5345

#include "te_uart.h"
#include "te_Si5345-Registers.h"

int si534x_init(unsigned char chip_addr){
	int i, Status;
	unsigned char val;
	unsigned short addr;

    // p_printf(("Si534x Init Start.\r\n"));
    // iic_init();
    p_printf(("Si534x Init Registers Write.\r\n"));

#ifdef SI5345_REVB_REG_CONFIG_NUM_REGS
    for (i=0; i<SI5345_REVB_REG_CONFIG_NUM_REGS; ++i) {
        val = si5345_revb_registers[i].value;
        addr = si5345_revb_registers[i].address;
        Status = iic_write16(chip_addr, addr, val);
        if(Status != XST_SUCCESS) {
            p_printf(("Couldn't write value 0x%02X to address 0x%04X.\n", val, addr));
            p_printf(("There were %d succesful I2C writes so far.\n", i));
            return Status;
        }
    }
#endif

#ifdef SI5345_REVD_REG_CONFIG_NUM_REGS
	for (i=0; i<SI5345_REVD_REG_CONFIG_NUM_REGS; ++i) {
		val = si5345_revd_registers[i].value;
		addr = si5345_revd_registers[i].address;
		Status = iic_write16(chip_addr, addr, val);
		if(Status != 0) {
			p_printf(("Couldn't write value 0x%02X to address 0x%04X.\n", val, addr));
			p_printf(("There were %d succesful I2C writes so far.\n", i));
			return Status;
		}
	}
#endif
	
    p_printf(("Si534x Init Complete.\r\n"));
#ifdef NVM_CODE
    p_printf(("Write this configuration to the Si5345 NVM? (y/N)\r\n"));
    if(uart_read_char() == 'y'){
        si534x_write_nvm(chip_addr)
    }
#endif
	return XST_SUCCESS;
}

#ifdef NVM_CODE
int si534x_write_nvm(unsigned char chip_addr){
	#ifndef NVM_REALY
	#error "To relase NVM write function please define NVM_REALY in te_si534x.h. You do it on your own risk!"
	#endif

#ifdef NVM_REALY
	int Status;
	unsigned char val;
	unsigned char active_nvm_bank;

    p_printf(("Si534x NVM Programming Start.\r\n"));

    Status = iic_read16(chip_addr, ACTIVE_NVM_BANK, &val);
    if(Status != XST_SUCCESS){
    	p_printf(("Error: Si534x Register read failed.\r\n"));
    	return Status;
    }
    active_nvm_bank = val;
    if(!((active_nvm_bank == 3) | (active_nvm_bank == 15))){
    	p_printf(("Error: Wrong ACTIVE_NVM_BANK d (should be 3 or 15).\r\n", active_nvm_bank));
    	return XST_FAILURE;
    }

    p_printf(("Write NVM_WRITE register.\r\n"));
    Status = iic_write16(chip_addr, NVM_WRITE, 0xC7);
    if(Status != XST_SUCCESS){
    	p_printf(("Error: Si534x Register write failed\r\n"));
    	return Status;
    }

    p_printf(("Poll DEVICE_READY.\r\n"));
    do{
    	Status = iic_read16(chip_addr, DEVICE_READY, &val);
    	if(Status != XST_SUCCESS){
    		p_printf(("Error: Si534x Register read failed\r\n"));
    		return Status;
    	}
    }
	while(val != 0x0F);

    p_printf(("Load the NVM contents into non-volatile memory.\r\n"));
    Status = iic_write16(chip_addr, NVM_READ_BANK, 0x01);
    if(Status != XST_SUCCESS){
    	p_printf(("Error: Si534x Register write failed\r\n"));
    	return Status;
    }

    p_printf(("Poll DEVICE_READY.\r\n"));
    do {
    	Status = iic_read16(chip_addr, DEVICE_READY, &val);
    	if(Status != XST_SUCCESS){
    		p_printf(("Error: Si534x Register read failed.\r\n"));
    		return Status;
    	}
    }
    while(val != 0x0F);

    p_printf(("Read ACTIVE_NVM_BANK.\r\n"));
    Status = iic_read16(chip_addr, ACTIVE_NVM_BANK, &val);
    if(Status != XST_SUCCESS){
    	p_printf(("Error: Si534x Register read failed.\r\n"));
    	return Status;
    }
    if(((active_nvm_bank << 2) | 0x03) != val){
    	p_printf(("Error: Wrong ACTIVE_NVM_BANK %d (was %d next should be %d).\r\n", val, active_nvm_bank, ((active_nvm_bank << 2) | 0x03)));
    	return XST_FAILURE;
    }

    p_printf(("Si534x NVM Programming Complete.\r\n"));
	#endif
	return XST_SUCCESS;
}
#endif
#endif /*Clock chip*/
